import java.awt.*;

public class MenuTest {
	public static void main(String[] args) {
		MainWindow w = new MainWindow();
		w.show();
	}
}

// Make a main window with two top-level menus: File and Help.
// Help has a submenu and demonstrates a few interesting menu items.
class MainWindow extends Frame {
	public MainWindow() {
		super("MenuTest Window");
		resize(200,200);

		// make a top level File menu
		FileMenu fileMenu = new FileMenu(this);

		// make a top level Help menu
		HelpMenu helpMenu = new HelpMenu(this);

		// make a menu bar for this frame 
		// and add top level menus File and Menu
		MenuBar mb = new MenuBar();
		mb.add(fileMenu);
		mb.add(helpMenu);
		setMenuBar(mb);
	}

	public void exit() {
		hide(); // hide the Frame
		dispose(); // tell windowing system to free resources
		System.exit(0); // exit
	}
}

// Encapsulate the look and behavior of the File menu
class FileMenu extends Menu {
	MainWindow mw;	// who owns us?
	public FileMenu(MainWindow m) {
		super("File");
		mw = m;
		add(new MenuItem("Open"));
		add(new MenuItem("Close"));
		add(new MenuItem("Exit"));
	}
	// respond to the Exit menu choice
	// This method does not pass unhandled events to the parent
	public boolean postEvent(Event e) {
		String item = (String) e.arg;
		if ( item.equals("Exit") ) mw.exit();
		else System.out.println("Selected FileMenu " + item);
		return true;
	}
}

// Encapsulate the look and behavior of the Help menu
class HelpMenu extends Menu {
	MainWindow mw;	// who owns us?
	public HelpMenu(MainWindow m) {
		super("Help");
		mw = m;
		add(new MenuItem("Fundamentals"));
		add(new MenuItem("Advanced"));
		add(new MenuItem("-"));
		add(new CheckboxMenuItem("Have Read The Manual"));
		add(new CheckboxMenuItem("Have Not Read The Manual"));

		// make a Misc sub menu of Help menu
		Menu subMenu = new Menu("Misc");
		subMenu.add(new MenuItem("Help!!!"));
		subMenu.add(new MenuItem("Why did that happen?"));
		add(subMenu);
	}
	// respond to a few menu items
	// This method does not pass unhandled events to the parent
	public boolean postEvent(Event e) {
		String item = (String) e.arg;
		if ( item.equals("Fundamentals") )
			System.out.println("Fundamentals");
		else if ( item.equals("Help!!!") ) System.out.println("Help!!!");
		// etc...
		return true;
	}
}
