/*****************************************************************************
* OS/2  PM/C Language Demo - MiniPIM                                         *
*                                                                            *
*       This demo is intended for comparison with the MiniPIM.java           *
*       Magercise.                                                           *
*                                                                            *
* Copyright 1997, <a href=http://www.MageLang.com>MageLang Institute</a>.    *
******************************************************************************/
#define INCL_WIN
#define INCL_GPI

#include <os2.h>                      /* PM header file                      */
#include <string.h>                   /* for strcpy, strcat                  */
#include "minipim.h"                  /* Resource symbolic identifiers       */
#include "pimedit.h"                  /* Function prototypes for Notepad     */
#include "pimphone.h"                 /* Function prototypes for Phone List  */
#include "pimcalc.h"                  /* Function prototypes for Calculator  */
#include "pimtodo.h"                  /* Function prototypes for To-Do List  */

/* Convenience function for creating buttons. */
static HWND CreateButton(HWND hwndParent, LONG Xpos, LONG Ypos,
                              LONG width, LONG height, PCHAR Caption, ULONG wndID);

HINI pimhini = NULLHANDLE;

CHAR AppName[] = "PIM_EXAMPLE";

/*
 * Function prototypes
 */
MRESULT EXPENTRY MyWindowProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 );

/* Window handles and other globals */
HAB  hab;                               /* PM anchor block handle       */
HWND hwndClient = NULLHANDLE;           /* Client area window handle    */
HWND hwndButton1 = NULLHANDLE;          /* Button 1 window handle       */
HWND hwndButton2 = NULLHANDLE;          /* Button 2 window handle       */
HWND hwndButton3 = NULLHANDLE;          /* Button 3 window handle       */
HWND hwndButton4 = NULLHANDLE;          /* Button 4 window handle       */
HWND hwndEditor = NULLHANDLE;           /* Editor window handle         */
HWND hwndPhone = NULLHANDLE;            /* Phone List window handle     */
HWND hwndCalc = NULLHANDLE;             /* Calculator window handle     */
HWND hwndToDo = NULLHANDLE;             /* To Do List window handle     */
HWND hwndActive = NULLHANDLE;           /* Which page is selected?      */
CHAR szEntry1[40] = "";
CHAR szWork[100] = "";
PSZ  pszErrMsg;


/*****************************************************/
/* Main program - create and initialize application  */
/*****************************************************/
INT main (VOID)
{
  HMQ  hmq;                             /* Message queue handle         */
  HWND hwndFrame = NULLHANDLE;          /* Frame window handle          */
  QMSG qmsg;                            /* Message from message queue   */
  ULONG flCreate;                       /* Window creation control flags*/

  if ((hab = WinInitialize(0)) == 0L) /* Initialize PM     */
     AbortProgram(hwndFrame, hwndClient); /* Terminate the application    */

  if ((hmq = WinCreateMsgQueue( hab, 0 )) == 0L)/* Create a msg queue */
     AbortProgram(hwndFrame, hwndClient); /* Terminate the application    */

  if (!WinRegisterClass(                /* Register window class        */
     hab,                               /* Anchor block handle          */
     (PSZ)"MyWindow",                   /* Window class name            */
     (PFNWP)MyWindowProc,               /* Address of window procedure  */
     CS_SIZEREDRAW,                     /* Class style                  */
     0                                  /* No extra window words        */
     ))
     AbortProgram(hwndFrame, hwndClient); /* Terminate the application    */

   if (!RegisterEditClass(hab))
     AbortProgram(hwndFrame, hwndClient); /* Terminate the application    */
   if (!RegisterPhoneClass(hab))
     AbortProgram(hwndFrame, hwndClient); /* Terminate the application    */
   if (!RegisterCalcClass(hab))
     AbortProgram(hwndFrame, hwndClient); /* Terminate the application    */
   if (!RegisterToDoClass(hab))
     AbortProgram(hwndFrame, hwndClient); /* Terminate the application    */

   flCreate = FCF_STANDARD &            /* Set frame control flags to   */
             ~FCF_SHELLPOSITION;        /* standard except for shell    */
                                        /* positioning.                 */
  /* Load the data from the ini file now for window creation */
  if ((pimhini = PrfOpenProfile(hab, "minipim.ini")) == NULLHANDLE)
     AbortProgram(hwndFrame, hwndClient); /* Terminate the application    */
  LoadToDoList(AppName, pimhini);
  LoadPhoneList(AppName, pimhini);

  if ((hwndFrame = WinCreateStdWindow(
               HWND_DESKTOP,            /* Desktop window is parent     */
               0,                       /* STD. window styles           */
               &flCreate,               /* Frame control flag           */
               "MyWindow",              /* Client window class name     */
               "",                      /* No window text               */
               0,                       /* No special class style       */
               (HMODULE)0L,             /* Resource is in .EXE file     */
               ID_WINDOW,               /* Frame window identifier      */
               &hwndClient              /* Client window handle         */
               )) == 0L)
     AbortProgram(hwndFrame, hwndClient); /* Terminate the application    */

    WinSetWindowText(hwndFrame, "MageLang OS/2 Example");

  if (!WinSetWindowPos( hwndFrame,      /* Shows and activates frame    */
                   HWND_TOP,            /* window at position 100, 100, */
                   100, 100, 500, 500,  /* and size 500, 500.           */
                   SWP_SIZE | SWP_MOVE | SWP_ACTIVATE | SWP_SHOW
                 ))
     AbortProgram(hwndFrame, hwndClient); /* Terminate the application    */

/*
 * Get and dispatch messages from the application message queue
 * until WinGetMsg returns FALSE, indicating a WM_QUIT message.
 */

  while( WinGetMsg( hab, &qmsg, 0L, 0, 0 ) )
    WinDispatchMsg( hab, &qmsg );

  /* Clear the profile file. */
  PrfWriteProfileData(pimhini, AppName, NULL, NULL, 0);

  /* And write out the new values. */
  SaveToDoList(AppName, pimhini);
  SavePhoneList(AppName, pimhini);

  /* Finally, close the file */
  PrfCloseProfile(pimhini);

  WinDestroyWindow(hwndFrame);           /* Tidy up...                   */
  WinDestroyMsgQueue( hmq );             /* Tidy up...                   */
  WinTerminate( hab );                   /* Terminate the application    */
  return 0;
} /* End of main */


/*****************************************************/
/* The main Window Procedure  for the program.       */
/* Sets up all of the controls for this application  */
/* and creates the subwindows (pages).               */
/*****************************************************/
MRESULT EXPENTRY MyWindowProc( HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  switch( msg )
  {
    case WM_CREATE:
      hwndButton1 = CreateButton(
                hwnd,                          /* Parent window          */
                10, 			       /* x position             */
                420,			       /* y position             */
                110, 			       /* width                  */
                30,			       /* height                 */
                "Notepad",                     /* Caption                */
                ID_BUTTON1);                   /* Window ID              */
      hwndButton2 = CreateButton(
                hwnd,                          /* Parent window          */
                125, 			       /* x position             */
                420,			       /* y position             */
                110, 			       /* width                  */
                30,			       /* height                 */
                "Phone List",                  /* Caption                */
                ID_BUTTON2);                   /* Window ID              */
      hwndButton3 = CreateButton(
                hwnd,                          /* Parent window          */
                240, 			       /* x position             */
                420,			       /* y position             */
                110, 			       /* width                  */
                30,			       /* height                 */
                "Calculator",                  /* Caption                */
                ID_BUTTON3);                   /* Window ID              */
      hwndButton4 = CreateButton(
                hwnd,                          /* Parent window          */
                355, 			       /* x position             */
                420,			       /* y position             */
                110, 			       /* width                  */
                30,			       /* height                 */
                "To-Do List",                  /* Caption                */
                ID_BUTTON4);                   /* Window ID              */

      /* Create the four "pages" of the application */
      hwndEditor = CreateEditWindow(hwnd);
      hwndPhone = CreatePhoneWindow(hwnd);
      hwndCalc = CreateCalcWindow(hwnd);
      hwndToDo = CreateToDoWindow(hwnd);

      hwndActive = hwndEditor;

      break;

    case WM_COMMAND:
      /*
       * When Exit is chosen, the application posts itself a WM_CLOSE
       * message.
       */
      {
         USHORT command;                   /* WM_COMMAND command value     */
         command = SHORT1FROMMP(mp1);      /* Extract the command value    */
         switch (command)
         {
           case ID_EXITPROG:
             WinPostMsg( hwnd, WM_CLOSE, (MPARAM)0, (MPARAM)0 );
             break;
           case ID_BUTTON1:
             WinShowWindow(hwndEditor,(BOOL)TRUE);
             WinShowWindow(hwndPhone,(BOOL)FALSE);
             WinShowWindow(hwndCalc,(BOOL)FALSE);
             WinShowWindow(hwndToDo,(BOOL)FALSE);
             hwndActive = hwndEditor;
             WinInvalidateRegion(hwnd, NULLHANDLE, TRUE);
             break;
           case ID_BUTTON2:
             WinShowWindow(hwndEditor,(BOOL)FALSE);
             WinShowWindow(hwndPhone,(BOOL)TRUE);
             WinShowWindow(hwndCalc,(BOOL)FALSE);
             WinShowWindow(hwndToDo,(BOOL)FALSE);
             hwndActive = hwndPhone;
             WinInvalidateRegion(hwnd, NULLHANDLE, TRUE);
             break;
           case ID_BUTTON3:
             WinShowWindow(hwndEditor,(BOOL)FALSE);
             WinShowWindow(hwndPhone,(BOOL)FALSE);
             WinShowWindow(hwndCalc,(BOOL)TRUE);
             WinShowWindow(hwndToDo,(BOOL)FALSE);
             hwndActive = hwndCalc;
             WinInvalidateRegion(hwnd, NULLHANDLE, TRUE);
             break;
           case ID_BUTTON4:
             WinShowWindow(hwndEditor,(BOOL)FALSE);
             WinShowWindow(hwndPhone,(BOOL)FALSE);
             WinShowWindow(hwndCalc,(BOOL)FALSE);
             WinShowWindow(hwndToDo,(BOOL)TRUE);
             hwndActive = hwndToDo;
             WinInvalidateRegion(hwnd, NULLHANDLE, TRUE);
             break;
           case ID_NEWFILE:
           case ID_OPENFILE:
           case ID_SAVEFILE:
           case ID_SAVEAS:
             WinSendMsg(hwndActive, msg, mp1, mp2);
             break;
           default:
             return WinDefWindowProc( hwnd, msg, mp1, mp2 );
         }

         break;
      }
    case WM_PAINT:
      /*
       * Window contents are drawn here in WM_PAINT processing.
       */
      {
         HPS    hps;                       /* Presentation Space handle    */
         RECTL  rc;                        /* Rectangle coordinates        */

         hps = WinBeginPaint( hwnd, 0L, &rc );
         WinFillRect( hps, &rc, SYSCLR_WINDOW);
         WinEndPaint( hps );                      /* Drawing is complete   */
         break;
      }
    case WM_ERASEBACKGROUND:
      /*
       * Return TRUE to request PM to paint the window background
       * in SYSCLR_WINDOW.
       */
      return (MRESULT)( TRUE );
    case WM_CLOSE:
      /*
       * This is the place to put termination routines
       */
      WinPostMsg( hwnd, WM_QUIT, (MPARAM)0,(MPARAM)0 );
      break;
    default:
      return WinDefWindowProc( hwnd, msg, mp1, mp2 );
  }
  return (MRESULT)FALSE;
} /* End of MyWindowProc */


/*****************************************************/
/* Display message box if error setting up           */
/*****************************************************/
VOID AbortProgram(HWND hwndFrame, HWND hwndClient)
{
   PERRINFO  pErrInfoBlk;
   PSZ       pszOffSet;
   void      stdprint(void);

   DosBeep(100,10);
   if ((pErrInfoBlk = WinGetErrorInfo(hab)) != (PERRINFO)NULL)
   {
      pszOffSet = ((PSZ)pErrInfoBlk) + pErrInfoBlk->offaoffszMsg;
      pszErrMsg = ((PSZ)pErrInfoBlk) + *((PSHORT)pszOffSet);
      if((INT)hwndFrame && (INT)hwndClient)
         WinMessageBox(HWND_DESKTOP,         /* Parent window is desk top */
                       hwndFrame,            /* Owner window is our frame */
                       (PSZ)pszErrMsg,       /* PMWIN Error message       */
                       "Error Msg",          /* Title bar message         */
                       MSGBOXID,             /* Message identifier        */
                       MB_MOVEABLE | MB_CUACRITICAL | MB_CANCEL ); /* Flags */
      WinFreeErrorInfo(pErrInfoBlk);
   }
   WinPostMsg(hwndClient, WM_QUIT, (MPARAM)NULL, (MPARAM)NULL);
} /* End */


/*****************************************************************************
* Convenience function to create buttons.
*****************************************************************************/
static HWND CreateButton(HWND hwndParent, LONG Xpos, LONG Ypos,
                         LONG width, LONG height, PCHAR Caption, ULONG wndID)
{
   HWND hwnd =
   WinCreateWindow(
                hwndParent,                    /* Parent window          */
                WC_BUTTON,                     /* Window class           */
                Caption,                       /* Window text            */
                WS_VISIBLE |
                BS_PUSHBUTTON,                 /* Button style           */
                Xpos, 			       /* x position             */
                Ypos,			       /* y position             */
                width,      		       /* Width                  */
                height,                        /* Height                 */
                hwndParent,                    /* Owner window           */
                HWND_TOP,                      /* Top of z-order         */
                wndID,                         /* Window identifier      */
                NULL,                          /* Control data           */
                NULL);                         /* Presentation parameters*/

   return hwnd;
}
