/*****************************************************************************
* OS/2  PM/C Language Demo - MiniPIM                                         *
*                                                                            *
*       This demo is intended for comparison with the MiniPIM.java           *
*       Magercise.                                                           *
*                                                                            *
* Copyright 1997, <a href=http://www.MageLang.com>MageLang Institute</a>.    *
******************************************************************************/
#define INCL_WIN
#define INCL_GPI
#define INCL_DOSFILEMGR
#define INCL_DOSERRORS

#include <os2.h>                        /* PM header file                */
#include <string.h>                     /* for strlen prototype          */
#include <stdio.h>                      /* for sscanf/sprintf prototypes */
#include "PIMCalc.h"                    /* Function prototypes           */

/* Convenience function for creating buttons */
static HWND CreateButton(HWND hwndParent, LONG Xpos, LONG Ypos,
                         LONG width, LONG height, PCHAR Caption, ULONG wndID);

/* Calculator button dimensions */
#define BW 60
#define BH 60
#define BX 120
#define BY 50

/* Button ID's */
#define ID_BUTTON0      350
#define ID_BUTTON1      351
#define ID_BUTTON2      352
#define ID_BUTTON3      353
#define ID_BUTTON4      354
#define ID_BUTTON5      355
#define ID_BUTTON6      356
#define ID_BUTTON7      357
#define ID_BUTTON8      358
#define ID_BUTTON9      359
#define ID_BUTTONPT     360
#define ID_BUTTONCLR    361
#define ID_BUTTONEQ     362
#define ID_BUTTONADD    363
#define ID_BUTTONSUB    364
#define ID_BUTTONMUL    365
#define ID_BUTTONDIV    366
#define ID_ENTRYFIELD   367

/* Handles for all of the windows */
static HWND hwndEntryField;             /* Entry Field window handle    */
static HWND hwndButton0 = NULLHANDLE;   /* Button 0 window handle       */
static HWND hwndButton1 = NULLHANDLE;   /* Button 1 window handle       */
static HWND hwndButton2 = NULLHANDLE;   /* Button 2 window handle       */
static HWND hwndButton3 = NULLHANDLE;   /* Button 3 window handle       */
static HWND hwndButton4 = NULLHANDLE;   /* Button 4 window handle       */
static HWND hwndButton5 = NULLHANDLE;   /* Button 5 window handle       */
static HWND hwndButton6 = NULLHANDLE;   /* Button 6 window handle       */
static HWND hwndButton7 = NULLHANDLE;   /* Button 7 window handle       */
static HWND hwndButton8 = NULLHANDLE;   /* Button 8 window handle       */
static HWND hwndButton9 = NULLHANDLE;   /* Button 9 window handle       */
static HWND hwndButtonPt = NULLHANDLE;  /* Button . window handle       */
static HWND hwndButtonClr = NULLHANDLE; /* Button C window handle       */
static HWND hwndButtonEq = NULLHANDLE;  /* Button = window handle       */
static HWND hwndButtonAdd = NULLHANDLE; /* Button + window handle       */
static HWND hwndButtonSub = NULLHANDLE; /* Button - window handle       */
static HWND hwndButtonMul = NULLHANDLE; /* Button * window handle       */
static HWND hwndButtonDiv = NULLHANDLE; /* Button / window handle       */

/* Temporary storage for calculator data */
static double CurrentAmount = 0.0;
static double SavedAmount = 0.0;
static CHAR Operator = ' ';
static CHAR buffer[64];

/*****************************************************/
/* Register the Calculator window class              */
/*                                                   */
/* Keep the details of the class name local to this  */
/* module so we can change them independent from the */
/* main program.                                     */
/*****************************************************/
BOOL RegisterCalcClass(HAB hab)
{
   return WinRegisterClass(hab, (PSZ)"PIMCalc", (PFNWP)CalcWindowProc, CS_SIZEREDRAW, 0);
}

/*****************************************************/
/* Create the Calculator window                      */
/*                                                   */
/* The window is created as a child of the parent,   */
/* which is also the owner of this window.           */
/*****************************************************/
HWND CreateCalcWindow(HWND hwndParent)
{
   HWND hwnd = WinCreateWindow(hwndParent,
                               "PIMCalc",
                               "",
                               0,                    /* Initially invisible       */
                               0, 0, 495, 400,
                               hwndParent,
                               HWND_TOP,
                               ID_CALCWINDOW,
                               NULL,
                               NULL);
   return hwnd;
}

/*****************************************************/
/* Calculator Window Proc                            */
/*                                                   */
/* Handle events directed to the calculator window.  */
/*****************************************************/
MRESULT EXPENTRY CalcWindowProc(HWND hwndParent, ULONG msg, MPARAM mp1, MPARAM mp2 )
{
  LONG i;
  RECTL rcl;

  switch( msg )
  {
    case WM_CREATE:
      /* On creation, we build all of the button windows */
      hwndButton0 = CreateButton(
                hwndParent,                    /* Parent window          */
                BX, 			       /* x position             */
                BY,			       /* y position             */
                BW, 			       /* width                  */
                BH,			       /* height                 */
                "0",                           /* Caption                */
                ID_BUTTON0);                   /* Window ID              */
      hwndButtonPt = CreateButton(
                hwndParent,                    /* Parent window          */
                BX+(BW+5), 		       /* x position             */
                BY,			       /* y position             */
                BW, 			       /* width                  */
                BH,			       /* height                 */
                ".",                           /* Caption                */
                ID_BUTTONPT);                  /* Window ID              */
      hwndButtonClr = CreateButton(
                hwndParent,                    /* Parent window          */
                BX+2*(BW+5),		       /* x position             */
                BY+BH/2+1,		       /* y position             */
                BW, 			       /* width                  */
                BH/2-1,		               /* height                 */
                "C",                           /* Caption                */
                ID_BUTTONCLR);                 /* Window ID              */
      hwndButtonEq = CreateButton(
                hwndParent,                    /* Parent window          */
                BX+2*(BW+5),		       /* x position             */
                BY,		               /* y position             */
                BW, 			       /* width                  */
                BH/2-1, 	      	       /* height                 */
                "=",                           /* Caption                */
                ID_BUTTONEQ);                  /* Window ID              */
      hwndButtonAdd = CreateButton(
                hwndParent,                    /* Parent window          */
                BX+3*(BW+5),		       /* x position             */
                BY,			       /* y position             */
                BW, 			       /* width                  */
                BH,			       /* height                 */
                "+",                           /* Caption                */
                ID_BUTTONADD);                 /* Window ID              */
      hwndButton1 = CreateButton(
                hwndParent,                    /* Parent window          */
                BX, 			       /* x position             */
                BY+(BH+5),		       /* y position             */
                BW, 			       /* width                  */
                BH,			       /* height                 */
                "1",                           /* Caption                */
                ID_BUTTON1);                   /* Window ID              */
      hwndButton2 = CreateButton(
                hwndParent,                    /* Parent window          */
                BX+(BW+5), 		       /* x position             */
                BY+(BH+5),		       /* y position             */
                BW, 			       /* width                  */
                BH,			       /* height                 */
                "2",                           /* Caption                */
                ID_BUTTON2);                   /* Window ID              */
      hwndButton3 = CreateButton(
                hwndParent,                    /* Parent window          */
                BX+2*(BW+5), 		       /* x position             */
                BY+(BH+5),		       /* y position             */
                BW, 			       /* width                  */
                BH,			       /* height                 */
                "3",                           /* Caption                */
                ID_BUTTON3);                   /* Window ID              */
      hwndButtonSub = CreateButton(
                hwndParent,                    /* Parent window          */
                BX+3*(BW+5), 		       /* x position             */
                BY+(BH+5),		       /* y position             */
                BW, 			       /* width                  */
                BH,			       /* height                 */
                "-",                           /* Caption                */
                ID_BUTTONSUB);                 /* Window ID              */
      hwndButton4 = CreateButton(
                hwndParent,                    /* Parent window          */
                BX, 			       /* x position             */
                BY+2*(BH+5),		       /* y position             */
                BW, 			       /* width                  */
                BH,			       /* height                 */
                "4",                           /* Caption                */
                ID_BUTTON4);                   /* Window ID              */
      hwndButton5 = CreateButton(
                hwndParent,                    /* Parent window          */
                BX+(BW+5), 		       /* x position             */
                BY+2*(BH+5),		       /* y position             */
                BW, 			       /* width                  */
                BH,			       /* height                 */
                "5",                           /* Caption                */
                ID_BUTTON5);                   /* Window ID              */
      hwndButton6 = CreateButton(
                hwndParent,                    /* Parent window          */
                BX+2*(BW+5), 		       /* x position             */
                BY+2*(BH+5),		       /* y position             */
                BW, 			       /* width                  */
                BH,			       /* height                 */
                "6",                           /* Caption                */
                ID_BUTTON6);                   /* Window ID              */
      hwndButtonMul = CreateButton(
                hwndParent,                    /* Parent window          */
                BX+3*(BW+5), 		       /* x position             */
                BY+2*(BH+5),		       /* y position             */
                BW, 			       /* width                  */
                BH,			       /* height                 */
                "X",                           /* Caption                */
                ID_BUTTONMUL);                 /* Window ID              */
      hwndButton7 = CreateButton(
                hwndParent,                    /* Parent window          */
                BX, 			       /* x position             */
                BY+3*(BH+5),		       /* y position             */
                BW, 			       /* width                  */
                BH,			       /* height                 */
                "7",                           /* Caption                */
                ID_BUTTON7);                   /* Window ID              */
      hwndButton8 = CreateButton(
                hwndParent,                    /* Parent window          */
                BX+(BW+5), 		       /* x position             */
                BY+3*(BH+5),		       /* y position             */
                BW, 			       /* width                  */
                BH,			       /* height                 */
                "8",                           /* Caption                */
                ID_BUTTON8);                   /* Window ID              */
      hwndButton9 = CreateButton(
                hwndParent,                    /* Parent window          */
                BX+2*(BW+5), 		       /* x position             */
                BY+3*(BH+5),		       /* y position             */
                BW, 			       /* width                  */
                BH,			       /* height                 */
                "9",                           /* Caption                */
                ID_BUTTON9);                   /* Window ID              */
      hwndButtonDiv = CreateButton(
                hwndParent,                    /* Parent window          */
                BX+3*(BW+5), 			       /* x position             */
                BY+3*(BH+5),		       /* y position             */
                BW, 			       /* width                  */
                BH,			       /* height                 */
                "/",                           /* Caption                */
                ID_BUTTONDIV);                 /* Window ID              */
      hwndEntryField = WinCreateWindow(
                hwndParent,                    /* Parent window          */
                WC_ENTRYFIELD,                 /* Window class           */
                "0",                           /* Window text            */
                WS_VISIBLE |                   /* Make it visible        */
                ES_AUTOSCROLL |                /* Entry Field Control    */
                ES_MARGIN |                    /* Display a border       */
                ES_RIGHT,                      /* Right Justify horiz.   */
                BX, 			       /* x position             */
                BY+4*(BH+5),		       /* y position             */
                4*BW+15,      		       /* Width                  */
                30,                            /* Height                 */
                hwndParent,                    /* Owner window           */
                HWND_TOP,                      /* Top of z-order         */
                ID_ENTRYFIELD,                 /* Window identifier      */
                NULL,                          /* Control data           */
                NULL);                         /* Presentation parameters*/

      /* Init the calculator string buffer */
      buffer[0] = '\0';
      break;

    case WM_COMMAND:
      {
         USHORT command;                   /* WM_COMMAND command value     */
         command = SHORT1FROMMP(mp1);      /* Extract the command value    */

         /* Handle the button presses. If number, append to buffer,        */
         /* Otherwise, perform requested operation                         */
         switch (command)
         {
           case ID_BUTTON0:
             strcat(buffer,"0");
             WinSetWindowText(hwndEntryField, buffer);
             break;
           case ID_BUTTON1:
             strcat(buffer,"1");
             WinSetWindowText(hwndEntryField, buffer);
             break;
           case ID_BUTTON2:
             strcat(buffer,"2");
             WinSetWindowText(hwndEntryField, buffer);
             break;
           case ID_BUTTON3:
             strcat(buffer,"3");
             WinSetWindowText(hwndEntryField, buffer);
             break;
           case ID_BUTTON4:
             strcat(buffer,"4");
             WinSetWindowText(hwndEntryField, buffer);
             break;
           case ID_BUTTON5:
             strcat(buffer,"5");
             WinSetWindowText(hwndEntryField, buffer);
             break;
           case ID_BUTTON6:
             strcat(buffer,"6");
             WinSetWindowText(hwndEntryField, buffer);
             break;
           case ID_BUTTON7:
             strcat(buffer,"7");
             WinSetWindowText(hwndEntryField, buffer);
             break;
           case ID_BUTTON8:
             strcat(buffer,"8");
             WinSetWindowText(hwndEntryField, buffer);
             break;
           case ID_BUTTON9:
             strcat(buffer,"9");
             WinSetWindowText(hwndEntryField, buffer);
             break;
           case ID_BUTTONPT:
             strcat(buffer,".");
             WinSetWindowText(hwndEntryField, buffer);
             break;
           case ID_BUTTONCLR:                 /* The clear button */
             strcpy(buffer,"");
             SavedAmount = 0.0;
             CurrentAmount = 0.0;
             Operator = ' ';
             WinSetWindowText(hwndEntryField, "0");
             break;

           /* For any of the 4 operators, check what the last stored operator */
           /* was and evaluate that before storing the current one.           */
           case ID_BUTTONADD:
             sscanf(buffer, "%lf", &CurrentAmount);
             switch (Operator)
             {
                case '+': SavedAmount = SavedAmount+CurrentAmount; break;
                case '-': SavedAmount = SavedAmount-CurrentAmount; break;
                case '*': SavedAmount = SavedAmount*CurrentAmount; break;
                case '/': SavedAmount = SavedAmount/CurrentAmount; break;
                case ' ': SavedAmount = CurrentAmount; break;
             }
             Operator = '+';
             WinSetWindowText(hwndEntryField, buffer);
             buffer[0] = '\0';
             break;
           case ID_BUTTONSUB:
             sscanf(buffer, "%lf", &CurrentAmount);
             switch (Operator)
             {
                case '+': SavedAmount = SavedAmount+CurrentAmount; break;
                case '-': SavedAmount = SavedAmount-CurrentAmount; break;
                case '*': SavedAmount = SavedAmount*CurrentAmount; break;
                case '/': SavedAmount = SavedAmount/CurrentAmount; break;
                case ' ': SavedAmount = CurrentAmount; break;
             }
             Operator = '-';
             WinSetWindowText(hwndEntryField, buffer);
             buffer[0] = '\0';
             break;
           case ID_BUTTONMUL:
             sscanf(buffer, "%lf", &CurrentAmount);
             switch (Operator)
             {
                case '+': SavedAmount = SavedAmount+CurrentAmount; break;
                case '-': SavedAmount = SavedAmount-CurrentAmount; break;
                case '*': SavedAmount = SavedAmount*CurrentAmount; break;
                case '/': SavedAmount = SavedAmount/CurrentAmount; break;
                case ' ': SavedAmount = CurrentAmount; break;
             }
             Operator = '*';
             WinSetWindowText(hwndEntryField, buffer);
             buffer[0] = '\0';
             break;
           case ID_BUTTONDIV:
             sscanf(buffer, "%lf", &CurrentAmount);
             switch (Operator)
             {
                case '+': SavedAmount = SavedAmount+CurrentAmount; break;
                case '-': SavedAmount = SavedAmount-CurrentAmount; break;
                case '*': SavedAmount = SavedAmount*CurrentAmount; break;
                case '/': SavedAmount = SavedAmount/CurrentAmount; break;
                case ' ': SavedAmount = CurrentAmount; break;
             }
             Operator = '/';
             WinSetWindowText(hwndEntryField, buffer);
             buffer[0] = '\0';
             break;
           /* For the "=" key, just handle any pending operator and display the */
           /* resulting value.                                                  */
           case ID_BUTTONEQ:
             sscanf(buffer, "%lf", &CurrentAmount);
             switch (Operator)
             {
                case '+': CurrentAmount = SavedAmount+CurrentAmount; break;
                case '-': CurrentAmount = SavedAmount-CurrentAmount; break;
                case '*': CurrentAmount = SavedAmount*CurrentAmount; break;
                case '/': CurrentAmount = SavedAmount/CurrentAmount; break;
             }
             sprintf(buffer, "%lf", CurrentAmount);
             WinSetWindowText(hwndEntryField, buffer);
             Operator = ' ';
             break;
           default:
             return WinDefWindowProc( hwndParent, msg, mp1, mp2 );
         }
      break;
      }

    default:
      return WinDefWindowProc( hwndParent, msg, mp1, mp2 );
  }
  return (MRESULT)FALSE;
} /* End of CalcWindowProc */


/****************************************************************************
* Convenience function to create buttons.                                   *
*****************************************************************************/
static HWND CreateButton(HWND hwndParent, LONG Xpos, LONG Ypos,
                         LONG width, LONG height, PCHAR Caption, ULONG wndID)
{
   HWND hwnd =
   WinCreateWindow(
                hwndParent,                    /* Parent window          */
                WC_BUTTON,                     /* Window class           */
                Caption,                       /* Window text            */
                WS_VISIBLE |
                BS_PUSHBUTTON |
                BS_AUTOSIZE,                   /* Button style           */
                Xpos, 			       /* x position             */
                Ypos,			       /* y position             */
                width,      		       /* Width                  */
                height,                        /* Height                 */
                hwndParent,                    /* Owner window           */
                HWND_TOP,                      /* Top of z-order         */
                wndID,                         /* Window identifier      */
                NULL,                          /* Control data           */
                NULL);                         /* Presentation parameters*/

   return hwnd;
}
